import dataclasses

from sohstationviewer.model.reftek.reftek_reader.header import PacketHeader


@dataclasses.dataclass
class DTExtendedHeader:
    """
    The extended header of a DT packet.
    """
    event_number: int
    data_stream_number: int
    channel_number: int
    number_of_samples: int
    flags: int
    data_format: str


@dataclasses.dataclass
class DTPacket:
    """
    The decoded data of a DT packet.
    """
    header: PacketHeader
    extended_header: DTExtendedHeader
    data: int


@dataclasses.dataclass
class EHETExtendedHeader:
    """
    A collection of some useful information about an EH/ET packet. Technically,
    EH/ET packets do not have extended headers. We name this class what it is
    due to the way obspy.Reftek130 (and consequently, core.Reftek130) stores
    the data of processed packets. For more information, refer to
    Reftek130._data.
    """
    event_number: int
    data_stream_number: int
    flags: int
    data_format: str

    def __post_init__(self):
        self.channel_number = 0
        self.number_of_samples = 0


@dataclasses.dataclass
class EHETPacket:
    """
    The decoded data of an EH/ET packet. The extended_header field is to ensure
    compatibility with dt_packet.DTPacket. EH/ET packets do not have an
    extended header otherwise.
    """
    header: PacketHeader
    extended_header: EHETExtendedHeader
    data: bytes


@dataclasses.dataclass
class SOHExtendedHeader:
    """
    A collection of dummy data for some information needed so that
    core.Reftek130 can understand SOH packets.

    core.Reftek130 focuses on reading waveform data, so it wants information
    available in the waveform packets (EH/ET/DT). However, core.Reftek130 also
    supports SOH packets, which does not contain the required information. As
    a result, we need to store dummy data in its place.
    """
    event_number: int
    data_stream_number: int
    channel_number: int
    number_of_samples: int
    flags: int
    data_format: str


@dataclasses.dataclass
class SOHPacket:
    """
    The decoded data of an SOH packet. The extended_header field is to ensure
    compatibility with dt_packet.DTPacket. SOH packets do not have an
    extended header otherwise.
    """
    header: PacketHeader
    extended_header: SOHExtendedHeader
    data: bytes
